package com.oreilly.xml;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.DOMBuilder;

/**
 * <b><code>XmlRpcConfiguration</code></b> is a utility class
 *   that will load configuration information for XML-RPC servers
 *   and clients to use.
 * 
 * @author 
 *   <a href="mailto:brettmclaughlin@earthlink.net">Brett McLaughlin</a>
 * @version 1.0
 */
public class XmlRpcConfiguration {
  
    /** The stream to read the XML configuration from */
    private InputStream in;
  
    /** Port number server runs on */
    private int portNumber;
    
    /** Hostname server runs on */
    private String hostname;

    /** SAX Driver Class to load */
    private String driverClass;
    
    /** Handlers to register in XML-RPC server */
    private Hashtable handlers;
  
    /**
     * <p>
     * This will set a filename to read configuration
     *   information from.
     * </p>
     *
     * @param filename <code>String</code> name of
     *                 XML configuration file.
     */
    public XmlRpcConfiguration(String filename) 
        throws IOException {
        
        this(new FileInputStream(filename));
    }
    
    /**
     * <p>
     * This will set a filename to read configuration
     *   information from.
     * </p>
     *
     * @param in <code>InputStream</code> to read
     *           configuration information from.
     */
    public XmlRpcConfiguration(InputStream in) 
        throws IOException {    
            
        this.in = in;
        portNumber = 0;
        hostname = "";
        handlers = new Hashtable();
    
        // Parse the XML configuration information
        parseConfiguration();
    }
  
    /**
     * <p>
     * This returns the port number the server listens on.
     * </p>
     *
     * @return <code>int</code> number of server port.
     */
    public int getPortNumber() {
        return portNumber;
    }

    /**
     * <p>
     * This returns the hostname the server listens on.
     * </p>
     *
     * @return <code>String</code> hostname of server.
     */    
    public String getHostname() {
        return hostname;
    }

    /**
     * <p>
     * This returns the SAX driver class to load.
     * </p>
     *
     * @return <code>String</code> - name of SAX driver class.
     */
    public String getDriverClass() {
        return driverClass;
    }

    /**
     * <p>
     * This returns the handlers the server should register.
     * </p>
     *
     * @return <code>Hashtable</code> of handlers.
     */     
    public Hashtable getHandlers() {
        return handlers;
    }
  
    /**
     * <p>
     * Parse the XML configuration information and
     *   make it available to clients.
     * </p>
     *
     * @throws <code>IOException</code> when errors occur.
     */
    private void parseConfiguration() throws IOException {        
        try {
            // Request DOM Implementation and Xerces Parser
            DOMBuilder builder = 
                new DOMBuilder("org.jdom.adapters.XercesDOMAdapter");
                
            Namespace ns = Namespace.getNamespace("JavaXML", "http://www.oreilly.com/catalog/javaxml/");

            // Get the Configuration Document, with validation
            Document doc = builder.build(in);
            
            // Get the root element
            Element root = doc.getRootElement();             
            
            // Load the hostname, port, and handler class
            hostname = 
                root.getChild("hostname", ns).getText();
            driverClass = 
                root.getChild("parserClass", ns).getText();
            portNumber = 
                Integer.parseInt(
                    root.getChild("port", ns).getText());            
            
            // Get the handlers
            List handlerElements = 
                root.getChild("xmlrpc-server", ns)
                    .getChild("handlers", ns)
                    .getChildren("handler", ns);     

            Iterator i = handlerElements.iterator();
            while (i.hasNext()) {
                Element current = (Element)i.next();
                handlers.put(current.getChild("identifier", ns)
                                    .getText(),
                             current.getChild("class", ns)
                                    .getText());                  
            }
        } catch (JDOMException e) {
            throw new IOException(e.getMessage());
        }

    }
}