package com.oreilly.xml;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.DOMBuilder;
import org.jdom.output.XMLOutputter;

/**
 * <b><code>XmlRpcConfiguration</code></b> is a utility class
 *   that will load configuration information for XML-RPC servers
 *   and clients to use.
 * 
 * @author 
 *   <a href="mailto:brettmclaughlin@earthlink.net">Brett McLaughlin</a>
 * @version 1.0
 */
public class XmlRpcConfiguration {
  
    /** The stream to read the XML configuration from */
    private InputStream in;
  
    /** Port number server runs on */
    private int portNumber;
    
    /** Hostname server runs on */
    private String hostname;

    /** SAX Driver Class to load */
    private String driverClass;
    
    /** Handlers to register in XML-RPC server */
    private Hashtable handlers;

    /** JDOM Document tied to underlying XML */
    private Document doc;
  
    /**
     * <p>
     * This will set a filename to read configuration
     *   information from.
     * </p>
     *
     * @param filename <code>String</code> name of
     *                 XML configuration file.
     */
    public XmlRpcConfiguration(String filename) 
        throws IOException {
        
        this(new FileInputStream(filename));
    }
    
    /**
     * <p>
     * This will set a filename to read configuration
     *   information from.
     * </p>
     *
     * @param in <code>InputStream</code> to read
     *           configuration information from.
     */
    public XmlRpcConfiguration(InputStream in) 
        throws IOException {    
            
        this.in = in;
        portNumber = 0;
        hostname = "";
        handlers = new Hashtable();
    
        // Parse the XML configuration information
        parseConfiguration();
    }
  
    /**
     * <p>
     * This returns the port number the server listens on.
     * </p>
     *
     * @return <code>int</code> number of server port.
     */
    public int getPortNumber() {
        return portNumber;
    }

    /**
     * <p>
     * This will set the port number to listen to.
     * </p>
     *
     * @param portNumber <code>int</code> port to listen to.
     */
    public void setPortNumber(int portNumber) {
        this.portNumber = portNumber;
    }

    /**
     * <p>
     * This returns the hostname the server listens on.
     * </p>
     *
     * @return <code>String</code> hostname of server.
     */    
    public String getHostname() {
        return hostname;
    }

    /**
     * <p>
     * This will set the hostname for the server to listen to.
     * </p>
     *
     * @param hostname <code>String</code> name of server's host.
     */
    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    /**
     * <p>
     * This returns the SAX driver class to load.
     * </p>
     *
     * @return <code>String</code> - name of SAX driver class.
     */
    public String getDriverClass() {
        return driverClass;
    }

    /**
     * <p>
     * This will set the driver class for parsing.
     * </p>
     *
     * @param driverClass <code>String</code> name of parser class.
     */
    public void setDriverClass(String driverClass) {
        this.driverClass = driverClass;
    }    

    /**
     * <p>
     * This returns the handlers the server should register.
     * </p>
     *
     * @return <code>Hashtable</code> of handlers.
     */     
    public Hashtable getHandlers() {
        return handlers;
    }

    /**
     * <p>
     * This will set the handlers to register.
     * </p>
     *
     * @param handlers <code>Hashtable</code> of handler to register.
     */
    public void setHandlers(Hashtable handlers) {
        this.handlers = handlers;
    }
  
    /**
     * <p>
     * Parse the XML configuration information and
     *   make it available to clients.
     * </p>
     *
     * @throws <code>IOException</code> when errors occur.
     */
    private void parseConfiguration() throws IOException {        
        try {
            // Request DOM Implementation and Xerces Parser
            DOMBuilder builder = 
                new DOMBuilder("org.jdom.adapters.XercesDOMAdapter");
                
            Namespace ns = Namespace.getNamespace("JavaXML", "http://www.oreilly.com/catalog/javaxml/");

            // Get the Configuration Document, with validation
            doc = builder.build(in);
            
            // Get the root element
            Element root = doc.getRootElement();             
            
            // Load the hostname, port, and handler class
            hostname = 
                root.getChild("hostname", ns).getText();
            driverClass = 
                root.getChild("parserClass", ns).getText();
            portNumber = 
                Integer.parseInt(
                    root.getChild("port", ns).getText());            
            
            // Get the handlers
            List handlerElements = 
                root.getChild("xmlrpc-server", ns)
                    .getChild("handlers", ns)
                    .getChildren("handler", ns);     

            Iterator i = handlerElements.iterator();
            while (i.hasNext()) {
                Element current = (Element)i.next();
                handlers.put(current.getChild("identifier", ns)
                                    .getText(),
                             current.getChild("class", ns)
                                    .getText());                  
            }
        } catch (JDOMException e) {
            throw new IOException(e.getMessage());
        }
    }

    /**
     * <p>
     * This will save the current state out to the XML-RPC configuration 
     *   file.
     * </p>
     *
     * @throws <code>IOException</code> - when errors occur in saving.     
     */
    public synchronized void saveConfiguration(String filename)
        throws IOException {    
            
        saveConfiguration(new FileOutputStream(filename));
    }
    
    /**
     * <p>
     * This will save the current state out to the specified 
     *   <code>OutputStream</code>.
     * </p>
     *
     * @throws <code>IOException</code> - when errors occur in saving.
     */
    public synchronized void saveConfiguration(OutputStream out)
        throws IOException {
            
        Namespace ns = Namespace.getNamespace("JavaXML", "http://www.oreilly.com/catalog/javaxml/");
        Element root = doc.getRootElement();
            
        // Update the hostname
        root.getChild("hostname", ns)
            .setText(hostname);
               
        // Update the SAX driver class
        root.getChild("parserClass", ns)
            .setText(driverClass);
               
        // Update the port number
        root.getChild("port", ns)
            .setText(portNumber + "");

        // Easier to remove and re-add handlers
        Element handlersElement =
            root.getChild("xmlrpc-server", ns)
                .getChild("handlers", ns);                   
        handlersElement.removeChildren("handler", ns);
            
        // Add new handlers
        Enumeration handlerIDs = handlers.keys();
        while (handlerIDs.hasMoreElements()) {
            String handlerID = 
                (String)handlerIDs.nextElement();
                    
            // Ensure we don't register any blank string
            if (handlerID.trim().equals("")) {
                continue;
            }
                   
            String handlerClass =
                (String)handlers.get(handlerID);
                
            handlersElement.addContent(
                new Element("handler", ns)
                    .addContent(
                        new Element("identifier", ns)
                            .setText(handlerID))
                    .addContent(
                        new Element("class", ns)
                            .setText(handlerClass))
                    );
            }
                           
        // Output the document, use standard formatter
        XMLOutputter fmt = new XMLOutputter();
        fmt.output(doc, out);           
    }

    /**
     * <p>
     * This will save the current state out to the specified 
     *   <code>OutputStream</code>.
     * </p>
     *
     * @throws <code>IOException</code> - when errors occur in saving.
     */
/**
    public synchronized void saveConfiguration(OutputStream out)
        throws IOException {

        // Create the root element
        Namespace ns = Namespace.getNamespace("JavaXML", "http://www.oreilly.com/catalog/javaxml/");
        Element root = new Element("xmlrpc-config", ns);
        Document doc = new Document(root);
        doc.setDocType(new DocType("JavaXML:xmlrpc-config",
                                   "DTD/XmlRpc.dtd"));
                                   
        root.addContent(new Element("hostname", ns)
                .setText(hostname))
            .addContent(new Element("port", ns)
                .addAttribute("type", "unprotected")
                .setText(portNumber + ""))
            .addContent(new Element("parserClass", ns)
                .setText(driverClass));
                
        Element handlersElement = new Element("handlers", ns);
        Enumeration e = handlers.keys();
        while (e.hasMoreElements()) {
            String handlerID = (String)e.nextElement();
            String handlerClass = (String)handlers.get(handlerID);
            
            handlersElement.addContent(new Element("handler", ns)
                .addContent(new Element("identifier", ns)
                    .setText(handlerID))
                .addContent(new Element("class", ns)
                    .setText(handlerClass))
                );
        }
        
        root.addContent(new Element("xmlrpc-server", ns)
                .addContent(handlersElement));
                           
        // Output the document, use standard formatter
        XMLOutputter fmt = new XMLOutputter();
        fmt.output(doc, out);
    }
 */
}